import os.path
from math import pi
import substances


def get_density(temp, sal):
    """Calculate the density, using temperature and salinity."""
    cp = 5890.0 + 38.00 * temp - 0.3750 * temp * temp + 3.0 * sal
    clam = (1779.5 + 11.25 * temp - 0.0745 * temp * temp) - (3.8 + 0.01 * temp) * sal
    rho = 1000.0 * cp / (0.698 * cp + clam)
    return rho


def generate_files(target, parameters):
    """Write Delwaq input files in target, using parameters."""
    if parameters['S9'] != 'manual':
        d = dict(substances.predefined_substances)
        parameters['S1'] = d[parameters['S9']]
        parameters['S2'] = substances.predefined_substances_values[parameters['S9']][0]
        parameters['S3'] = substances.predefined_substances_values[parameters['S9']][1]
    # Create delwaq_inp_load.inc
    with open(os.path.join(target, 'delwaq_inp_load.inc'), 'w') as output_file:
        output_file.write(r"ITEM"+"\n")
        output_file.write(r" 'Pipe'"+"\n")
        output_file.write(r"CONCENTRATIONS"+"\n")
        output_file.write(r"   USEFOR 'FLOW' 'FLOW'"+"\n")
        output_file.write(r"   USEFOR 'Continuity' 'Continuity'"+"\n")
        output_file.write(r"   USEFOR 'Substance' 'Substance'"+"\n")
        output_file.write(r"TIME LINEAR"+"\n")
        output_file.write(r"DATA"+"\n")
        output_file.write(r" 'FLOW' 'Continuity' 'Substance'"+"\n")
        output_file.write(r"2011/01/01-00:00:00  " + str(parameters['D4']/3600.) + r"  0.0000e+000  " + str(parameters['D8'])+"\n")
        output_file.write(r"2011/01/02-00:00:00  " + str(parameters['D4']/3600.) + r"  0.0000e+000  " + str(parameters['D8'])+"\n")
    # Create (multiple) iskenderun_?_delwaq_inp_params.inc
    for i in range(parameters['E7']):
        with open(os.path.join(target, 'iskenderun_'+str(i)+'_delwaq_inp_params.inc'), 'w') as output_file:
            output_file.write(r"CONSTANTS 'k_Substance' DATA " + str(parameters['S2'])+"\n")
            output_file.write(r"CONSTANTS 'Temp' DATA " + str(parameters['E14_'+str(i)])+"\n")
            output_file.write(r"CONSTANTS 'SS' DATA " + str(parameters['E12_'+str(i)])+"\n")
            output_file.write(r"CONSTANTS 'kd_Substance' DATA " + str(parameters['S3'])+"\n")
    # Create Integrate_isk.ini
    with open(os.path.join(target, 'Integrate_isk.ini'), 'w') as output_file:
        output_file.write(r"2                                           ; nr of Substances"+"\n")
        output_file.write(r";   substance      Kd (m3/g)"+"\n")
        output_file.write(r"     Continuity  0"+"\n")
        output_file.write(r"     Substance   " + str(parameters['S4'])+"\n")
        output_file.write(str(parameters['E7'])+r"                                           ; nr of Condition"+"\n")
        output_file.write(r"; %  SS (g/m3)   file"+"\n")
        for i in range(parameters['E7']):
            output_file.write(str(parameters['E08_'+str(i)]) + r"   " + str(parameters['E12_'+str(i)]) + "    iskenderun_"+str(i)+"_res.map"+"\n")
    # Create (multiple) iskenderun_?.scr
    rho_ref = 0
    if 'D5' in parameters and 'D6' in parameters:
        rho_ref = get_density(parameters['D5'], parameters['D6'])
    for i in range(parameters['E7']):
        with open(os.path.join(target, 'iskenderun_'+str(i)+'.scr'), 'w') as output_file:
            output_file.write(r"[Input]"+"\n")
            if str(parameters['E1']) == 'open':
                output_file.write(r"Case="+str(1)+"\n")
            if str(parameters['E1']) == 'enclosed':
                output_file.write(r"Case="+str(2)+"\n")
            output_file.write(r"Name="+str(parameters['S1'])+"\n")
            output_file.write(r"Length="+str(parameters['E2'])+"\n")
            output_file.write(r"Width1="+str(parameters['E3'])+"\n")
            output_file.write(r"GridDir="+str(parameters['E4'])+"\n")
            output_file.write(r"TotalDepth=" + str(parameters['E5']+parameters['E3']*parameters['E6'])+"\n")
            output_file.write(r"ShoreDepth="+str(parameters['E5'])+"\n")
            output_file.write(r"NLong="+str(max(20, min(100, int(parameters['E2']/50.))))+"\n")
            output_file.write(r"NLat="+str(max(20, min(100, int(parameters['E3']/50.))))+"\n")
            output_file.write(r"NVert=6"+"\n")
            output_file.write(r"WindDir=" + str(parameters['E10_'+str(i)])+"\n")
            output_file.write(r"WindSpeed=" + str(parameters['E09_'+str(i)])+"\n")
            output_file.write(r"CurSpeed=" + str(parameters['E11_'+str(i)])+"\n")
            output_file.write(r"LDischarge=" + str(parameters['D1'])+"\n")
            output_file.write(r"WDischarge=" + str(parameters['D2'])+"\n")
            output_file.write(r"SpecifiedZDischarge=" + str(parameters['D3'])+"\n")
            rho = get_density(parameters['E13_'+str(i)], parameters['E14_'+str(i)])
            if 'D5' not in parameters and 'D6' not in parameters:
                rho_ref = rho
            if abs(rho - rho_ref) < 1.:
                output_file.write(r"MidFieldZDischarge="+'UNCHANGED'+"\n")
            elif rho > rho_ref:
                output_file.write(r"MidFieldZDischarge="+'BOTTOM'+"\n")
            else:
                output_file.write(r"MidFieldZDischarge="+'SURFACE'+"\n")
    # Create Integrate2.ini
    with open(os.path.join(target, 'Integrate2.ini'), 'w') as output_file:
        output_file.write(r"10                   ; number of distances for Jet3D"+"\n")
        output_file.write(r"0 10 20 30 40 50 60 70 80 90"+"\n")
        output_file.write(r"15                   ; number of distances for Delwaq"+"\n")
        output_file.write(r"100 200 300 400 500 600 700 800 900 1000 1200 1400 1600 1800 2000"+"\n")
        output_file.write(r"2                                           ; nr of Substances"+"\n")
        output_file.write(r";   substance      Kd (m3/kg)   Cd (g/m3)    SSd (g/m3)   Cb (g/m3)  EQS_MAC (g/m3)   MZ_MAC (m)  EQS_AA (g/m3)   MZ_AA (m)   SafetyFactor"+"\n")
        output_file.write(r"     Continuity    0.0         0.0          0.0          0.0        1.0              "+str(parameters['S6'])+r"        1.0             "+str(parameters['S8'])+r"        "+str(parameters['O1'])+r"         "+"\n")
        output_file.write(r"     Substance     "+str(parameters['S3'])+r"        "+str(parameters['D8'])+r"         "+str(parameters['D9'])+r"        "+str(parameters['S4'])+r"       "+str(parameters['S5'])+r"             "+str(parameters['S6'])+r"        "+str(parameters['S7'])+r"            "+str(parameters['S8'])+r"        "+str(parameters['O1'])+r""+"\n")
        output_file.write(str(parameters['E7'])+r"                                           ; nr of Condition"+"\n")
        output_file.write(r"; %  SS (g/m3)   file"+"\n")
        for i in range(parameters['E7']):
            output_file.write(str(parameters['E08_'+str(i)])+' '+str(parameters['E12_'+str(i)])+"   iskenderun_"+str(i)+"_res.map   jet3d_"+str(i)+".tek"+"\n")


def generate_additional_files(target, parameters):
    """Write Jet3d input files in target, using parameters."""
    # Create (multiple) jet3d_?.inp
    for i in range(parameters['E7']):
        xxx = ''
        with open(os.path.join(target, 'iskenderun_'+str(i)+'.uds'), 'r') as input_file:
            line = input_file.readline()
            xxx = float(line.strip().split()[0])
        with open(os.path.join(target, 'jet3d_'+str(i)+'.inp'), 'w') as output_file:
            output_file.write(r"      100.       10.                    ; Range van Y (defaults) (2F10.0)"+"\n")
            output_file.write(r"    0    1    1                         ; Vlag voor constante of variabele snelheid (default: 0), NY en NZ allebei 1 (3I5)"+"\n")
            output_file.write('{:.8f}'.format(xxx)+r"                              ; constante SNELHEID van het ontvangende water (m/s) (absolute waarde)"+"\n")
            output_file.write(r"    0    1                              ; Vlag voor constante of variabele dichtheid (default: 0), NYR 1 (2I5)"+"\n")
            output_file.write(r"1000.                                   ; density ontvangende water default (kg/m3)"+"\n")
            output_file.write(r"        0.        0.        5.          ; X (arbitrair) Y (default) en Z (default) van lozing (0 is bij de bodem) (3F10.0)"+"\n")
            output_file.write(r"        0.                              ; S0 (initiele afstand default           (F10.0)"+"\n")
            output_file.write(r"      100.       90.        0.          ; 3F10.0 C0 ,THETA0,PHI0 (initiele CONCENTRATIE in de pluim, theta is hor hoek (default), phi is verticale hoek default"+"\n")
            zzz = parameters['D4'] / (900. * pi * parameters['D7'] * parameters['D7'])
            output_file.write('{:.8f}'.format(parameters['D7'])+' {:.8f}'.format(zzz)+r"     1000.          ; 3F10.0 BR0,U0    ,RHO0 DIAMETER van de pijp in m, SNELHEID van de pluim in m/s, dichtheid lozing in kg/m3 default"+"\n")
            output_file.write(r"      150.        4.                    ; 2F10.0 SMAX,PRSTEP begrenzing domein (in m default) stapgrootte in m defaults"+"\n")
