import shutil
import os
import subprocess
import threading
import tempfile
import platform


from . import delwaq_input


class Counter:
    def __init__(self):
        self.i = 0
        # create a lock
        self.lock = threading.Lock()

    def __iter__(self):
        return self

    def next(self):
        # acquire/release the lock when updating self.i
        with self.lock:
            self.i += 1
            return self.i

counter = Counter()


def do_run(appstruct):
    """Copy the template directory, edit the input files and run the calculation."""
    # target directory must not yet exist.
    unique_number = counter.next()
    current_dir = os.path.split(os.path.abspath(__file__))[0]
    target_dir = tempfile.mkdtemp(prefix='elveqo_', suffix="_%s" % (unique_number,))
    shutil.rmtree(target_dir)  # copytree wants to make target_dir by itself.
    shutil.copytree(os.path.join(current_dir, 'delwaq/Template'), target_dir)
    # Generate Delwaq input files
    delwaq_input.generate_files(target_dir, appstruct)
    # run the calculation
    if platform.system() == 'Windows':
        cwdir = os.path.join(target_dir, 'bin', 'win32')
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(cwdir, 'nepflow.exe'),
                             'iskenderun_'+str(i)+'.scr', 'iskenderun_'+str(i)+'.hyd'], cwd=target_dir)
        # Generate additional input files
        delwaq_input.generate_additional_files(target_dir, appstruct)
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(cwdir, 'jet3d.exe'),
                             'jet3d_'+str(i)+'.inp', 'jet3d_'+str(i)+'.tek'], cwd=target_dir)
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(os.path.abspath(target_dir), 'run_delwaq.bat'),
                             'iskenderun_'+str(i)], cwd=target_dir)
        subprocess.call([os.path.join(cwdir, 'integrate2.exe'),
                         'integrate2.ini', 'iskenderun_0.lga', 'scremo_isk2.map', 'scremo_isk_pp2.csv'], cwd=target_dir)
    elif platform.system() == 'Linux':
        cwdir = os.path.join(target_dir, 'bin', 'lnx')
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(cwdir, 'NepFlow'),
                             'iskenderun_'+str(i)+'.scr', 'iskenderun_'+str(i)+'.hyd'], cwd=target_dir,
                             env={'LD_LIBRARY_PATH': '/opt/gcc48/lib64:$LD_LIBRARY_PATH'})
        # Generate additional input files
        delwaq_input.generate_additional_files(target_dir, appstruct)
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(cwdir, 'Jet3D'),
                             'jet3d_'+str(i)+'.inp', 'jet3d_'+str(i)+'.tek'], cwd=target_dir,
                             env={'LD_LIBRARY_PATH': '/opt/gcc48/lib64:$LD_LIBRARY_PATH'})
        for i in range(appstruct['E7']):
            subprocess.call([os.path.join(os.path.abspath(target_dir), 'run_delwaq.sh'),
                             'iskenderun_'+str(i)], cwd=target_dir)
        subprocess.call([os.path.join(cwdir, 'Integrate2'),
                         'Integrate2.ini', 'iskenderun_0.lga', 'scremo_isk2.map', 'scremo_isk_pp2.csv'],
                         cwd=target_dir, env={'LD_LIBRARY_PATH': '/opt/gcc48/lib64:$LD_LIBRARY_PATH'})
    # read the results
    results = dict()
    with open(os.path.join(target_dir, 'scremo_isk_pp2.csv'), 'r') as result_file:
        lines = result_file.readlines()
        for number, line in enumerate(lines[9:19]):
            parts = [part.strip() for part in line.split(';')]
            results['row'+str(number)] = [parts[0]]
            results['row'+str(number)].append(parts[2])
            results['row'+str(number)].append(parts[1])
        for number, line in enumerate(lines[23:38]):
            parts = [part.strip() for part in line.split(';')]
            results['row'+str(number+10)] = [parts[0]]
            results['row'+str(number+10)].append(parts[2])
            results['row'+str(number+10)].append(parts[1])
        replace_names = ['EQS', 'defined mixing zone', 'concentration at edge of defined mixing zone', 'discharge permitted?']
        for number, line in enumerate(lines[40:44]):
            parts = [part.strip() for part in line.split(';')]
            parts[0] = replace_names[number]
            results['row'+str(number+25)] = [parts[0]]
            results['row'+str(number+25)].append(parts[2])
            results['row'+str(number+25)].append(parts[1])
    # clean-up
    shutil.rmtree(target_dir)
    # return results for rendering
    return results
