import colander
import deform
from pkg_resources import resource_filename
import os.path
import substances


class elveqo_schema(colander.MappingSchema):
    """Schema for the parameter definitions."""
    # The discharge definitions.
    D1 = colander.SchemaNode(colander.Float(),
                              title = 'Position of discharge: distance X1 along the coast, relative to point R [m]',
                              description = 'Position of discharge along the coast.')
    D2 = colander.SchemaNode(colander.Float(),
                             title = 'Position of discharge: distance X2 from the coast [m]',
                             description = 'Position of discharge from the coast.')
    D3 = colander.SchemaNode(colander.Float(),
                             default = 0,
                             validator = colander.Range(0., 100.),
                             title = 'Position of discharge: depth from water surface [m]',
                             description = 'Depth of discharge, relative to water surface. [0, 100]')
    D4 = colander.SchemaNode(colander.Float(),
                             validator = colander.Function(lambda x: 0. < x <= 10000.),
                             title = 'Discharge flow rate [m3/h]',
                             description = 'Discharge flow rate. <0, 10000]')
    D7 = colander.SchemaNode(colander.Float(),
                             default = 0.3,
                             validator = colander.Range(0.05, 5.),
                             title = 'Discharge pipe opening diameter [m]',
                             description = 'Discharge pipe opening diameter. [0.05, 5]')
    D5 = colander.SchemaNode(colander.Float(),
		             missing = colander.drop,
                             validator = colander.Range(10., 40.),
                             title = 'Discharge temperature [degree Celcius]',
                             description = 'Discharge temperature. [10, 40]')
    D6 = colander.SchemaNode(colander.Float(),
		             missing = colander.drop,
                             validator = colander.Range(0., 100.),
                             title = 'Discharge salinity [ppt]',
                             description = 'Discharge salinity. [0, 100]')
    D8 = colander.SchemaNode(colander.Float(),
                             validator = colander.Function(lambda x: 0. < x <= 1000.),
                             title = 'Discharge concentration-chemical of concern [g/m3]',
                             description = 'Discharge concentration for the chemical of concern. <0, 1000]')
    D9 = colander.SchemaNode(colander.Float(),
                             validator = colander.Range(0., 1000.),
                             default = 0.,
                             title = 'Discharge concentration of suspended solids [g/m3]',
                             description = 'Discharge concentration of suspended solids. [0, 1000]')
    # The substance definitions.
    S9 = colander.SchemaNode(colander.String(),
                             default = 'manual',
                             widget = deform.widget.SelectWidget(values=substances.predefined_substances),
                             title = 'Predefined substance',
                             description = 'Choose a predefined substance, when applicable. Specifies default values for fields Name, Substance decay rate and Substance partition coefficient.')
    S1 = colander.SchemaNode(colander.String(),
                             missing = colander.drop,
                             title = 'Name',
                             description = 'Name of the chemical of concern.')
    S2 = colander.SchemaNode(colander.Float(),
                             missing = colander.drop,
                             validator = colander.Range(0., 100.),
                             title = 'Substance decay rate [1/d]',
                             description = 'Substance decay rate. [0, 100]')
    S3 = colander.SchemaNode(colander.Float(),
                             missing = colander.drop,
                             validator = colander.Range(0., 100000.),
                             title = 'Substance partition coefficient [m3/kg]',
                             description = 'Substance partition coefficient. [0, 100000]')
    S4 = colander.SchemaNode(colander.Float(),
                             default = 0.,
                             validator = colander.Range(0., 1000.),
                             title = 'Background concentration [g/m3]',
                             description = 'Background concentration. [0, 1000]')
    S5 = colander.SchemaNode(colander.Float(),
                             validator = colander.Function(lambda x: 0. < x <= 10.),
                             title = 'EQS (MAC, maximum allowable concentration) [g/m3]',
                             description = 'Environmental Quality Standard for MAC. <0, 10]')
    S6 = colander.SchemaNode(colander.Float(),
                             default = 500.,
                             validator = colander.Range(5., 2500.),
                             title = 'Allowable mixing zone (MAC) [m]',
                             description = 'The allowable mixing zone for MAC. [5, 2500] ')
    S7 = colander.SchemaNode(colander.Float(),
                             validator = colander.Function(lambda x: 0. < x <= 10.),
                             title = 'EQS (AA, annually averaged concentration) [g/m3]',
                             description = 'Environmental Quality Standard for AA. <0, 10].')
    S8 = colander.SchemaNode(colander.Float(),
                             default = 500.,
                             validator = colander.Range(5., 2500.),
                             title = 'Allowable mixing zone (AA) [m]',
                             description = 'The allowable mixing zone for AA. [5, 2500]')
    # The environment input variable definitions.
    E1 = colander.SchemaNode(colander.String(),
                             default = 'open',
                             widget = deform.widget.SelectWidget(values=(('open','open'), ('enclosed','enclosed'))),
                             title = 'Type of environment (open / semi-enclosed)',
                             description = 'Open represents a straight coastline and three open water sides. Semi-enclosed respresents an environment with just one open water side.')
    E2 = colander.SchemaNode(colander.Float(),
                             default = 5000.,
                             validator = colander.Range(1000., 5000.),
                             title = 'Size of simulation domain L1 along the coast [m]',
                             description = 'Size of simulation domain along the coast. [1000, 5000]')
    E3 = colander.SchemaNode(colander.Float(),
                             default = 2000.,
                             validator = colander.Range(500., 5000.),
                             title = 'Size of simulation domain L2 perpendicular to the coast [m]',
                             description = 'Size of simulation domain perpendicular to the coast. [500, 5000]')
    E4 = colander.SchemaNode(colander.Float(),
                             default = 0.,
                             validator = colander.Range(0., 360.),
                             title = 'Orientation of study area relative to North [degree]',
                             description = 'Orientation of study area relative to North.')
    E5 = colander.SchemaNode(colander.Float(),
                             validator = colander.Range(1., 100.),
                             title = 'Water depth near-shore [m]',
                             description = 'Water depth in the simulation domain, immediately adjacent to the (inner) coastline. In case of stratification this value specifies the depth of the pycnocline. [1, 100]')
    E6 = colander.SchemaNode(colander.Float(),
                             default = 0.,
                             validator = colander.Range(0., 1.),
                             title = 'Bottom slope [m/m]',
                             description = 'The increase in water depth (in meters) when moving one meter away from the coastline. [0, 1]')
    E7 = colander.SchemaNode(colander.Int(),
                             default = 1,
                             validator = colander.Range(1, 10),
                             title = 'Nr of different meteo / current alternatives',
                             description = 'The number of different meteo and/or current alternatives. The number specified here relates to the number of columns (as shown below) that need to be specified / will be taken into account. [1, 10]')
    E08_0 = colander.SchemaNode(colander.Float(),
                                default = 100.,
                                validator = colander.Function(lambda x: 0. < x <= 100.),
                                title = 'Frequency of occurrence [%]',
                                description = 'The frequency of occurence for this meteo / current alternative. The sum of all frequencies should always equal 100. <0, 100]')
    E08_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    E08_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Function(lambda x: 0. < x <= 100.))
    #
    E09_0 = colander.SchemaNode(colander.Float(),
                                default = 5.,
                                validator = colander.Range(0., 40.),
                                title = 'Wind speed at 10 m above water surface [m/s]',
                                description = ' The wind speed at 10 m above water surface. [0, 40]')
    E09_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    E09_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 40.))
    #
    E10_0 = colander.SchemaNode(colander.Float(),
                                default = 0.,
                                validator = colander.Range(0., 360.),
                                title = 'Wind direction relative to North [degree]',
                                description = 'The wind direction relative to North. [0, 360]')
    E10_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    E10_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 360.))
    #
    E11_0 = colander.SchemaNode(colander.Float(),
                                default = 0.05,
                                validator = colander.Range(-1.0, 1.0),
                                title = 'Current speed [m/s]',
                                description = 'The current speed. [-1, 1]')
    E11_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    E11_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(-1., 1.))
    #
    E12_0 = colander.SchemaNode(colander.Float(),
                                default = 0.,
                                validator = colander.Range(0., 100.),
                                title = 'Suspended solids concentration [g/m3]',
                                description = 'Suspended solids concentration. [0, 100]')
    E12_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    E12_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(0., 100.))
    #
    E13_0 = colander.SchemaNode(colander.Float(),
                                default = 37.,
                                validator = colander.Range(20., 50.),
                                title = 'Salinity (above pycnocline) [ppt]',
                                description = 'Salinity (above pycnocline). [20, 50]')
    E13_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    E13_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(20., 50.))
    #
    E14_0 = colander.SchemaNode(colander.Float(),
                                default = 20.,
                                validator = colander.Range(10., 40.),
                                title = 'Temperature (above pycnocline) [degree Celcius]',
                                description = 'Temperature (above pycnocline). [10, 40]')
    E14_1 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_2 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_3 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_4 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_5 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_6 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_7 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_8 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    E14_9 = colander.SchemaNode(colander.Float(), missing = colander.drop, title='', validator = colander.Range(10., 40.))
    #
    O1 = colander.SchemaNode(colander.Float(),
                             default = 2.,
                             validator = colander.Range(2., 10.), 
                             title = 'Safety factor Fate & Transport',
                             description = 'The safety factor for Fate & Transport. Please refer to the Guidance Document for more details. [2, 10]')

schema = elveqo_schema()

# Override the default renderer for only the deform.Form instance "form".
deform_templates = resource_filename('deform', 'templates')
current_dir = os.path.split(os.path.abspath(__file__))[0]
search_path = (os.path.join(current_dir, 'templates'), deform_templates)
input_form_renderer = deform.ZPTRendererFactory(search_path)

#form = deform.Form(schema, renderer=input_form_renderer, buttons=('submit', 'save', 'load'))
