import os.path
import tempfile
import pickle
import shutil

from math import pi
from pyramid.view import view_config
from pyramid.response import FileResponse
from pyramid import renderers
import deform
import colander
import peppercorn

from . import schemas
from .models import do_run, counter


# Consider using temporarydirectory for automatic cleanup
# http://stackoverflow.com/questions/19296146/with-tempfile-temporarydirectory


def validate_input(appstruct, form, controls):
    error_found = False
    predefined_substance = False
    if appstruct['S9'] != 'manual':
        predefined_substance = True
    if 'S1' in appstruct and predefined_substance:
        exc = colander.Invalid(form['S1'], 'Predefined substance selected. Do not specify a name manually.')
        form['S1'].error = exc
        error_found = True 
    elif 'S1' not in appstruct and not predefined_substance:
        exc = colander.Invalid(form['S1'], 'Specify a name for manually defined substance.')
        form['S1'].error = exc
        error_found = True 
    if 'S2' in appstruct and predefined_substance:
        exc = colander.Invalid(form['S2'], 'Predefined substance selected. Do not specify a value manually.')
        form['S2'].error = exc
        error_found = True 
    elif 'S2' not in appstruct and not predefined_substance:
        exc = colander.Invalid(form['S2'], 'Specify a value for manually defined substance.')
        form['S2'].error = exc
        error_found = True 
    if 'S3' in appstruct and predefined_substance:
        exc = colander.Invalid(form['S3'], 'Predefined substance selected. Do not specify a value manually.')
        form['S3'].error = exc
        error_found = True 
    elif 'S3' not in appstruct and not predefined_substance:
        exc = colander.Invalid(form['S3'], 'Specify a value for manually defined substance.')
        form['S3'].error = exc
        error_found = True 
    if appstruct['D1'] < 0:
        exc = colander.Invalid(form['D1'], 'Value too small.')
        form['D1'].error = exc
        error_found = True
    if appstruct['D2'] < 0:
        exc = colander.Invalid(form['D2'], 'Value too small.')
        form['D2'].error = exc
        error_found = True
    if appstruct['D1'] > appstruct['E2']:
        exc = colander.Invalid(form['D1'], 'Value larger than simulation domain L1.')
        form['D1'].error = exc
        error_found = True
    if appstruct['D2'] > appstruct['E3']:
        exc = colander.Invalid(form['D2'], 'Value larger than simulation domain L2.')
        form['D2'].error = exc
        error_found = True
    if appstruct['D2'] >= (appstruct['E3'] - 2 * appstruct['S6']):
        msg = 'Distance between discharge location and open boundary should be at least two mixing zone distances.'
        exc = colander.Invalid(form['D2'], msg)
        form['D2'].error = exc
        exc = colander.Invalid(form['S6'], msg)
        form['S6'].error = exc
        error_found = True
    if appstruct['D2'] >= (appstruct['E3'] - 2 * appstruct['S8']):
        msg = 'Distance between discharge location and any open boundary should be at least two mixing zone distances.'
        exc = colander.Invalid(form['D2'], msg)
        form['D2'].error = exc
        exc = colander.Invalid(form['S8'], msg)
        form['S8'].error = exc
        error_found = True
    if appstruct['E1'] == 'open':
        if not(2 * appstruct['S6'] < appstruct['D1'] < appstruct['E2'] - 2 * appstruct['S6']):
            msg = 'Distance between discharge location and any open boundary should be at least two mixing zone distances.'
            exc = colander.Invalid(form['D1'], msg)
            form['D1'].error = exc
            exc = colander.Invalid(form['S6'], msg)
            form['S6'].error = exc
            error_found = True
        if not(2 * appstruct['S8'] < appstruct['D1'] < appstruct['E2'] - 2 * appstruct['S8']):
            msg = 'Distance between discharge location and any open boundary should be at least two mixing zone distances.'
            exc = colander.Invalid(form['D1'], msg)
            form['D1'].error = exc
            exc = colander.Invalid(form['S8'], msg)
            form['S8'].error = exc
            error_found = True
    if appstruct['D3'] > appstruct['E5'] + appstruct['D2'] * appstruct['E6']:
        exc = colander.Invalid(form['D3'], 'Discharge below seabed.')
        form['D3'].error = exc
        error_found = True
    if (appstruct['D4'] / (900. * pi * appstruct['D7'] * appstruct['D7'])) < 0.05:
        exc = colander.Invalid(form['D4'], 'Value too small compared to discharge pipe opening diameter.')
        form['D4'].error = exc
        exc = colander.Invalid(form['D7'], 'Value too large compared to discharge flow rate.')
        form['D7'].error = exc
        error_found = True
    if ('D5' not in appstruct and 'D6' in appstruct) or \
       ('D5' in appstruct and 'D6' not in appstruct):
        msg = 'Define value for both discharge temperature and salinity, or leave them both empty.'
        exc = colander.Invalid(form['D5'], msg)
        form['D5'].error = exc
        exc = colander.Invalid(form['D6'], msg)
        form['D6'].error = exc
        error_found = True
    if appstruct['S4'] > appstruct['S5']:
        exc = colander.Invalid(form['S4'], 'Value must not exceed EQS (MAC).')
        form['S4'].error = exc
        exc = colander.Invalid(form['S5'], 'Value smaller than background concentration.')
        form['S5'].error = exc
        error_found = True
    if appstruct['S4'] > appstruct['S7']:
        exc = colander.Invalid(form['S4'], 'Value must not exceed EQS (AA).')
        form['S4'].error = exc
        exc = colander.Invalid(form['S7'], 'Value smaller than background concentration.')
        form['S7'].error = exc
        error_found = True
    if appstruct['E7'] > 1:
        cnl = ('08', '09', '10', '11', '12', '13', '14')
        sum_percentage = float(appstruct['E08_0'])
        for i in xrange(0, 7):
            for j in xrange(1, int(appstruct['E7'])):
                tv = 'E'+cnl[i]+'_'+str(j)
                if tv not in appstruct:
                    exc = colander.Invalid(form[tv], 'Required')
                    form[tv].error = exc
                    error_found = True
                elif i == 0:
                    sum_percentage += float(appstruct[tv])
        if sum_percentage != 100.:
            for j in xrange(0, int(appstruct['E7'])):
                tv = 'E08_'+str(j)
                exc = colander.Invalid(form[tv], 'Sum of percentages should be 100')
                form[tv].error = exc
                exc = colander.Invalid(form['E7'], 'Check sum of percentages for this number of alternatives.')
                form['E7'].error = exc
            error_found = True
    if appstruct['E7'] == 1 and float(appstruct['E08_0']) != 100.:
        exc = colander.Invalid(form['E08_0'], 'Sum of percentages should be 100')
        form['E08_0'].error = exc
        exc = colander.Invalid(form['E7'], 'Check sum of percentages for this number of alternatives.')
        form['E7'].error = exc
        error_found = True
    if error_found:
        cstruct = form.deserialize(peppercorn.parse(controls))
        raise deform.ValidationFailure(form, cstruct, None)


@view_config(route_name='input', renderer='templates/inputtemplate.pt')
def input_view(request):
    form = deform.Form(schemas.schema, renderer=schemas.input_form_renderer, buttons=('submit', 'save', 'load'))

    if 'save' in request.POST:
        controls = request.POST.items()
        data_string = pickle.dumps(controls)
        unique_number = counter.next()
        target_dir = tempfile.mkdtemp(prefix='elveqo_', suffix="_%s" % (unique_number,))
        with open(os.path.join(target_dir, 'form_data.txt'), 'wb') as form_data_file:
            form_data_file.write(data_string)
        res = FileResponse(os.path.join(target_dir, 'form_data.txt'))
        res.content_disposition = 'attachment; filename="form_data.txt"'
        return res
    if 'load' in request.POST:
        if not hasattr(request.POST['fdata'], 'file'):
            return {'form': form.render()}
        input_file = request.POST['fdata'].file
        unique_number = counter.next()
        target_dir = tempfile.mkdtemp(prefix='elveqo_', suffix="_%s" % (unique_number,))
        with open(os.path.join(target_dir, 'form_data.txt'), 'wb') as pickle_file:
            input_file.seek(0)
            while True:
                data = input_file.read(2 << 16)
                if not data:
                    break
                pickle_file.write(data)
        controls = pickle.load(open(os.path.join(target_dir, 'form_data.txt'), 'rb'))
        shutil.rmtree(target_dir)
        try:
            appstruct = form.validate(controls)  # throws exception if form data is incomplete!
            form.set_appstruct(appstruct)
        except deform.ValidationFailure, e:
            return {'form': e.render()}
    if 'submit' in request.POST:
        controls = request.POST.items()
        try:
            # reset all previous errors
            for child in form.children:
                child.error = None
            appstruct = form.validate(controls)
            validate_input(appstruct, form, controls)
            calculation_results = do_run(appstruct)
            return {'form': renderers.render('templates/output.pt', calculation_results, request)}
        except deform.ValidationFailure, e:
            return {'form': e.render()}
    return {'form': form.render()}
